#lang racket

;; Data Definition
(define-struct foo (field1 left right))
(define-struct bar (lo hi))

;; A Baz is one of
;; -- (make-foo Number Baz Baz)
;; -- (make-bar Number Number)
#|
(define (baz-fn b)
  (cond
    [(foo? b) (...
                (foo-data b)
                (foo-left b)
                (foo-right b))]
    [(baz? b) (...
                (baz-lo b)
                (baz-hi b))]))
|#
;; a sample function that follows the baz template:

;; Baz Number -> Number
(define (baz-mymin b n)
  (cond
    [(foo? b) (min
                (foo-field1 b)
                (baz-mymin (foo-left b) n)
                (baz-mymin (foo-right b) n)
                (other-fn b n))]
    [(bar? b) (min  
                n (bar-lo b) (bar-hi b))]))

;;;;;;; convert this data definition to an interface and classes.

(define Baz<%>
  (interface ()
    ;; list here all the functions that follow the baz template

    ;; mymin : Number -> Number
    ;; the method we are translating
    mymin  

    ;; other-method : Number -> Number
    ;; some other method on baz's
    other-method

    ))

;; use class* to define a class that satisfies a given interface

;; a Foo% is a (make-foo [first-one Number][left Baz][right Baz])
(define Foo%
  (class* object% (baz<%>)
    (init-field
      first-one                         ; Number
      left                              ; Baz
      right                             ; Baz
      )

    (super-new)

    ;; mymin : Number -> Number
    ;; STRATEGY: Structural Decomposition on this
    (define/public (mymin n)
      (min
        field1
        (send left mymin n)
        (send right mymin n)
        (send this other-method n)))

    (define/public (other-method n) ...)))


;; a Bar% is a (make-bar [lo Number][hi Number])
(define Bar%
  (class* object% (baz<%>)
    (init-field lo hi)                  ; both are Numbers

    (super-new)

    ;; mymin : Number -> Number
    ;; STRATEGY: domain knowledge
    (define/public (mymin n)
      (min  
        n lo hi))

    (define/public (other-method n) ...)

    ))






    