#lang racket

;; This illustrates the use of interfaces to hide object
;; implementations

(define StupidRobot<%>
  (interface ()

    ;; a new StupidRobot<%> is required to start at position 0
    
    ;; -> StupidRobot<%>
    ;; RETURNS: a Robot just like this one, except moved one position
    ;; to the right
    move-right

    ;; -> Integer
    ;; RETURNS: the current x-position of the robot
    get-pos

    ))

;; Here are 3 implementations of StupidRobot<%>.

;; represent the position by a field x.
(define Robot1%
  (class* object% (StupidRobot<%>)

    (init-field [x 0])  ;; interp: the position of the robot.

    (super-new)

    (define/public (move-right)
      (new Robot1% [x (+ x 1)]))

    (define/public (get-pos)
      x)

    ))


(define Robot2%
  (class* object% (StupidRobot<%>)

    (init-field [y 0])   ;; interp: the negative of the position of
                         ;; the robot.

    (super-new)

    (define/public (move-right)
      (new Robot2% [y (- y 1)]))

    (define/public (get-pos)
      (- y))

    ))


(define Robot3%
  (class* object% (StupidRobot<%>)

    (init-field [x empty])    ;; a list whose length is equal to the
                              ;; position of the robot

    ;; Puzzle: if the robot could move left, and therefore reach
    ;; negative positions, how could you adapt this representation to
    ;; handle this?

    (super-new)

    (define/public (move-right)
      (new Robot3% [x (cons 1 x)]))

    (define/public (get-pos)
      (length x))

    ))

;; -> StupidRobot<%>
(define (new-robot)
  (local
    ((define i (random 2)))
    (cond
      [(= i 0) (new Robot1%)]
      [(= i 1) (new Robot2%)]
      [(= i 3) (new Robot3%)])))